/*BK based on a simple systray applet example by Rodrigo De Castro, 2007*/
/*sys tray applet to monitor free personal storage space
  GPL license /usr/share/doc/legal/gpl-2.0.txt.
  freememapplet_tray is started in /usr/sbin/delayedrun and pfbpanel.
  2.1: changed from png to included xpm images, passed param not needed.
  2.2: 100517: always read free space in save file, not RAM space.
  2.2: pet pkg has moved executable to /root/Startup.
  2.3: 100820: 01micko: PUPMODE==2 fix.
  2.3.1: BK 110805 testing with PUPMODE=2, needs fix, there is no /dev/root.
   (puppy has a fixed df that returns actual partition instead of /dev/root)
  2.4 (20120519): rodin.s: added gettext.
  */

#include <string.h>
#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gstdio.h>
#include <sys/statfs.h>

#define _(STRING)    gettext(STRING)

GtkStatusIcon *tray_icon;
unsigned int interval = 10000; /*update interval in milliseconds*/
char memdisplayfree[16];
char memdisplaytotal[16];
unsigned long long sizetotal;
unsigned long long sizefree;
unsigned long long sizefreeprev = 0;
char memdisplaylong[256];
char save_layer_dir[256];
unsigned int ptoffset;
char *ptsecond;
int percentfree;

GdkPixbuf *grey_pixbuf;
GdkPixbuf *critical_pixbuf;
GdkPixbuf *ok_pixbuf;
GdkPixbuf *good_pixbuf;
GdkPixbuf *excellent_pixbuf;

GError *gerror = NULL;

gboolean Update(gpointer ptr);

int getFileSystemData(const char* fln)
{
	struct statfs vfs;
	int retval;
	retval = statfs(fln, &vfs);
	if (retval == 0)
	{
		sizetotal = vfs.f_blocks;
		sizetotal = (sizetotal * vfs.f_bsize) / 1048576;
		sizefree = vfs.f_bavail;
		sizefree = (sizefree * vfs.f_bsize) / 1048576;
	}
	return retval;
}

gboolean Update(gpointer ptr) {
	
    //read free personal storage...
	if (getFileSystemData(save_layer_dir) != 0)
		sizetotal = sizefree = 0;

    if (sizefreeprev == sizefree) return; //unchanged.
    sizefreeprev=sizefree;
    
    //format display...
    if ( sizefree < 1000 ) sprintf(memdisplayfree, "%lldM", sizefree);
    else if (sizefree >= 10240) sprintf(memdisplayfree, "%lldG", (sizefree/1024)); //>=10G
    else sprintf(memdisplayfree, "%.1fG", (float)(sizefree/1024.0));
    if ( sizetotal < 1000 ) sprintf(memdisplaytotal, "%lldM", sizetotal);
    else if (sizetotal >= 10240) sprintf(memdisplaytotal, "%lldG", (sizetotal/1024)); //>=10G
    else sprintf(memdisplaytotal, "%.1fG", (float)(sizetotal/1024.0));
    
    //update tooltip...
    memdisplaylong[0]=0;
    strcat (memdisplaylong,memdisplaytotal);
    strcat (memdisplaylong,_(" personal storage, free space "));
    strcat (memdisplaylong,memdisplayfree);
    gtk_status_icon_set_tooltip(tray_icon, memdisplaylong);

    //update icon... (sizefree,sizetotal are in MB)
    if (gtk_status_icon_get_blinking(tray_icon)==TRUE) gtk_status_icon_set_blinking(tray_icon,FALSE);
    percentfree=(sizefree*100)/sizetotal;
    if (sizefree < 20) {
        gtk_status_icon_set_from_pixbuf(tray_icon,critical_pixbuf);
        gtk_status_icon_set_blinking(tray_icon,TRUE);
    }
    else if (sizefree < 50) {
        gtk_status_icon_set_from_pixbuf(tray_icon,critical_pixbuf);
    }
    else if (percentfree < 20) {
        gtk_status_icon_set_from_pixbuf(tray_icon,critical_pixbuf);
    }
    else if (percentfree < 45) {
        gtk_status_icon_set_from_pixbuf(tray_icon,ok_pixbuf);
	}
    else if (percentfree < 70) {
        gtk_status_icon_set_from_pixbuf(tray_icon,good_pixbuf);
	}
    else {
        gtk_status_icon_set_from_pixbuf(tray_icon,excellent_pixbuf);
	}
}

void tray_icon_on_click(GtkStatusIcon *status_icon, gpointer user_data)
{
    system("partview &");
}

void tray_icon_on_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data)
{
    printf("Popup menu\n");
}

static GtkStatusIcon *create_tray_icon() {

    tray_icon = gtk_status_icon_new();
    g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(tray_icon_on_click), NULL);
    g_signal_connect(G_OBJECT(tray_icon), "popup-menu", G_CALLBACK(tray_icon_on_menu), NULL);

	grey_pixbuf=gdk_pixbuf_new_from_file("/usr/share/pixmaps/puppy/container_0.svg",&gerror);
    critical_pixbuf=gdk_pixbuf_new_from_file("/usr/share/pixmaps/puppy/container_1.svg",&gerror);
    ok_pixbuf=gdk_pixbuf_new_from_file("/usr/share/pixmaps/puppy/container_2.svg",&gerror);
    good_pixbuf=gdk_pixbuf_new_from_file("/usr/share/pixmaps/puppy/container_3.svg",&gerror);
    excellent_pixbuf=gdk_pixbuf_new_from_file("/usr/share/pixmaps/puppy/container_4.svg",&gerror);

    gtk_status_icon_set_visible(tray_icon, TRUE);

    return tray_icon;
}

int main(int argc, char **argv) {
    int len;
    GtkStatusIcon *tray_icon;
	FILE *fp;
	char line_buffer[1024];

    setlocale( LC_ALL, "" );
    bindtextdomain( "freememapplet_tray", "/usr/share/locale" );
    textdomain( "freememapplet_tray" );
    
	save_layer_dir[0] = 0;
	fp = fopen("/etc/rc.d/PUPSTATE", "r");
	if (fp != NULL)
	{
		while (fgets(line_buffer, sizeof line_buffer, fp))
		{
			// strip trailing '\n' if it exists
			len = strlen(line_buffer) - 1;
			while ((len >= 0) && (isspace(line_buffer[len])))
				line_buffer[len--] = 0;
			if (strncmp(line_buffer, "SAVE_LAYER=", 11) == 0)
			{
				strcpy(save_layer_dir, "/initrd");
				strcat(save_layer_dir, line_buffer + 12); //ignore leading '
				len = strlen(save_layer_dir) - 1;
				if (len >= 0)
					save_layer_dir[len] = 0; //strip trailing '
				break;
			}
		}
		fclose(fp);
	}

    gtk_init(&argc, &argv);
    tray_icon = create_tray_icon();
        
    gtk_timeout_add(interval, Update, NULL);
    Update(NULL);

    gtk_main();

    return 0;
}
